package lt.marsrutai.mobile.serialize;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import lt.marsrutai.serialize.exception.SerializeException;

import com.dyuproject.protostuff.LinkedBuffer;
import com.dyuproject.protostuff.ProtostuffIOUtil;
import com.dyuproject.protostuff.Schema;
import com.dyuproject.protostuff.runtime.RuntimeSchema;

public class ProtostuffSerializer implements Serializer {

    @SuppressWarnings("rawtypes")
    private Map<String, Schema> requestToSchemaMap = new HashMap<String, Schema>();

    private List<String> requestsReturningObject;

    @SuppressWarnings("unchecked")
    @Override
    public byte[] writeObject(SerializeRequest request) {

        if (!requestToSchemaMap.containsKey(request.getName())) {
            throw new SerializeException("Unknown object name: " + request.getName());
        }

        List<Object> objectList = null;
        if (requestsReturningObject.contains(request.getName())) {
            objectList = new ArrayList<Object>();
            objectList.add(request.getObject());
        }
        else {
            objectList = (List<Object>) request.getObject();
        }

        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            GZIPOutputStream gos = new GZIPOutputStream(bos);

            ProtostuffIOUtil.writeListTo(gos, objectList, requestToSchemaMap.get(request.getName()), LinkedBuffer.allocate(256));

            gos.flush();
            gos.close();
            return bos.toByteArray();
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public Object readObject(DeserializeRequest request) {

        if (!requestToSchemaMap.containsKey(request.getName())) {
            throw new SerializeException("Unknown object name: " + request.getName());
        }

        try {
            ByteArrayInputStream bis = new ByteArrayInputStream(request.getArray());
            GZIPInputStream gis = new GZIPInputStream(bis);

            Object object = ProtostuffIOUtil.parseListFrom(gis, requestToSchemaMap.get(request.getName()));

            if (requestsReturningObject.contains(request.getName())) {
                List<Object> objectList = (List<Object>) object;
                if (null != objectList && objectList.size() == 1) {
                    object = objectList.get(0);
                }
            }

            return object;
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

    @SuppressWarnings("rawtypes")
    public void configure(Map<String, Class<?>> requestToClassMap, List<String> requestsReturningObject) {
        requestToSchemaMap = new HashMap<String, Schema>(requestToClassMap.size());
        for (Map.Entry<String, Class<?>> entry: requestToClassMap.entrySet()) {
            requestToSchemaMap.put(entry.getKey(), RuntimeSchema.getSchema(entry.getValue()));
        }

        this.requestsReturningObject = requestsReturningObject;
    }

}
